package com.calypso.bildroid;

import java.io.IOException;

import org.jsoup.Jsoup;
import org.jsoup.nodes.Document;
import org.jsoup.select.Elements;

import android.util.Log;

//********************************************************************
//News.java       Author: Celal
//
//Represents an item.
//********************************************************************
public class Article {

	/* VARIABLES */

	private static final String TAG = "Article";

	/**
	 * String title
	 * 
	 * A variable to hold the article's title
	 */
	private String title;

	/**
	 * String content
	 * 
	 * A variable to hold the article's content
	 */
	private String content;

	/**
	 * String articleURL
	 * 
	 * A variable to hold the article's url address
	 */
	private String articleURL;

	/**
	 * String imageURL
	 * 
	 * A variable to hold the url of images that belongs the article
	 */
	private String imageURL;

	/* CONSTRUCTOR */

	/**
	 * Constructor for data coming from network
	 * 
	 * @param articleURL
	 *            - an article's URL adress from BilNews
	 */
	public Article(String articleURL) {

		this.articleURL = articleURL;
		
		// For debug purpose
		final long startLocal = System.currentTimeMillis();

		// Store a null reference in the variable doc
		Document doc = null;

		// fetch the specified URL and parse to a HTML DOM
		try {
			doc = Jsoup.connect(articleURL).get();
		} catch (IOException e) {
			Log.e(TAG, "Error during Jsoup.connect(" + articleURL + ").get()");
		}

		if (doc != null) {
			Elements titles = doc.select("h3");
			Elements entry = doc.select("div.entry");
			Elements image = doc.select("img");

			if (!titles.isEmpty()) {
				this.title = titles.first().text().trim();
			} else {
				this.title = null;
			}

			if (!entry.isEmpty()) {
				this.content = entry.first().text().trim()
						.substring(this.title.length());
			} else {
				this.content = null;
			}

			if (!image.isEmpty()) {
				this.imageURL = image.first().attr("abs:src");
			} else {
				this.imageURL = null;
			}
			
            Log.d(TAG, "Fetching this article took " + (System.currentTimeMillis() - startLocal) + "ms");

			//Send an INFO log message.
			Log.i(TAG, "URL : " + this.articleURL);
			Log.i(TAG, "Title : " + this.title);
			Log.i(TAG, "Content : " + this.content);
			Log.i(TAG, "Image URL : " + this.imageURL);

		}

	}

	/**
	 * Constructor for data coming from database.
	 * 
	 * @param title
	 * @param content
	 * @param articleURL
	 * @param imageURL
	 */
	public Article(String title, String content, String articleURL,
			String imageURL) {
		this.title = title;
		this.content = content;
		this.articleURL = articleURL;
		this.imageURL = imageURL;
	}

	/* METHODS */

	/**
	 * String getTitle()
	 * 
	 * @return the title
	 */
	public String getTitle() {
		return this.title;
	}

	/**
	 * void setTitle(String title)
	 * 
	 * @param title
	 *            the title to set
	 */
	public void setTitle(String title) {
		this.title = title;
	}

	/**
	 * String getContent()
	 * 
	 * @return the content
	 */
	public String getContent() {
		return this.content;
	}

	/**
	 * void setContent(String content)
	 * 
	 * @param content
	 *            the content to set
	 */
	public void setContent(String content) {
		this.content = content;
	}

	/**
	 * String getArticleURL()
	 * 
	 * @return the articleURL
	 */
	public String getArticleURL() {
		return this.articleURL;
	}

	/**
	 * void setArticleURL(String articleURL)
	 * 
	 * @param articleURL
	 *            the articleURL to set
	 */
	public void setArticleURL(String articleURL) {
		this.articleURL = articleURL;
	}

	/**
	 * String getImageURL()
	 * 
	 * @return the imageURL
	 */
	public String getImageURL() {
		return this.imageURL;
	}

	/**
	 * void setImageURL(String imageURL)
	 * 
	 * @param imageURL
	 *            the imageURL to set
	 */
	public void setImageURL(String imageURL) {
		this.imageURL = imageURL;
	}

}
