package com.calypso.bildroid;

import java.io.FilterInputStream;
import java.io.IOException;
import java.io.InputStream;

import org.apache.http.HttpEntity;
import org.apache.http.HttpResponse;
import org.apache.http.HttpStatus;
import org.apache.http.client.methods.HttpGet;

import android.app.Activity;
import android.content.Intent;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.net.http.AndroidHttpClient;
import android.os.Bundle;
import android.util.Log;
import android.view.View;
import android.widget.Button;
import android.widget.ImageView;
import android.widget.TextView;

public class ArticleDetailedView extends Activity {
	/* VARIABLES */

	// a String constant to store the class file name that is used in Log
	// methods.
	private static final String TAG = "ArticleDetailedView";

	private TextView tvForContent;
	private ImageView ivForPic;
	private TextView tvForTitle;
	private Button backButton;

	/* METHODS */

	@Override
	public void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setContentView(R.layout.news_detailed_view);

		// find textview that will print the title in the UI by its ID
		tvForTitle = (TextView) findViewById(R.id.newsDVTitle);

		// find textview that will print the content in the UI by its ID
		tvForContent = (TextView) findViewById(R.id.dnewsDVContent);

		// find imageview that will display the image in the UI by its ID
		ivForPic = (ImageView) findViewById(R.id.newsDVImage);

		// find button that will finish this activity in the UI by its ID
		backButton = (Button) findViewById(R.id.newsDVBackButton);

		final Intent anIntent = getIntent();

		if (anIntent.hasExtra("title")
				&& anIntent.getStringExtra("title") != null) {
			tvForTitle.setText(anIntent.getStringExtra("title"));
		} else {
			tvForTitle.setText("Title Not Found");
		}

		if (anIntent.hasExtra("content")
				&& anIntent.getStringExtra("content") != null) {
			tvForContent.setText(anIntent.getStringExtra("content"));
		} else {
			tvForContent.setText("Text Not Found");
		}

		if (anIntent.hasExtra("imageURL")
				&& anIntent.getStringExtra("imageURL") != null) {
			new Thread(new Runnable() {
				public void run() {
					final Bitmap bitmap = downloadBitmap(anIntent.getStringExtra(
							"imageURL"));
					ivForPic.post(new Runnable() {
						public void run() {
							ivForPic.setImageBitmap(bitmap);
						}
					});
				}
			}).start();
		} else {
			ivForPic.setVisibility(View.GONE);
		}
	}

	@Override
	protected void onStart() {
		super.onStart();
		// The activity is about to become visible.
	}

	@Override
	protected void onResume() {
		super.onResume();
		// The activity has become visible (it is now "resumed").
	}

	@Override
	protected void onPause() {
		super.onPause();
		// Another activity is taking focus (this activity is about to be
		// "paused").
	}

	@Override
	protected void onStop() {
		super.onStop();
		// The activity is no longer visible (it is now "stopped")
	}

	@Override
	protected void onDestroy() {
		super.onDestroy();
		// The activity is about to be destroyed.
	}

	private Bitmap downloadBitmap(String url) {
		if (url != null) {
			final AndroidHttpClient client = AndroidHttpClient
					.newInstance("Android");
			final HttpGet getRequest = new HttpGet(url);

			try {
				HttpResponse response = client.execute(getRequest);
				final int statusCode = response.getStatusLine().getStatusCode();
				if (statusCode != HttpStatus.SC_OK) {
					Log.w(TAG, "Error " + statusCode
							+ " while retrieving bitmap from " + url);
					return null;
				}

				final HttpEntity entity = response.getEntity();
				if (entity != null) {
					InputStream inputStream = null;
					try {
						inputStream = entity.getContent();
						final Bitmap bitmap = BitmapFactory.decodeStream(
								new FlushedInputStream(inputStream));
						return bitmap;
					} finally {
						if (inputStream != null) {
							inputStream.close();
						}
						entity.consumeContent();
					}
				}
			} catch (IOException e) {
				getRequest.abort();
				Log.w(TAG, "I/O error while retrieving bitmap from " + url, e);
			} catch (IllegalStateException e) {
				getRequest.abort();
				Log.w(TAG, "Incorrect URL: " + url);
			} catch (Exception e) {
				getRequest.abort();
				Log.w(TAG, "Error while retrieving bitmap from " + url, e);
			} finally {
				if ((client instanceof AndroidHttpClient)) {
					((AndroidHttpClient) client).close();
				}
			}
		}
		return null;

	} // end of downloadBitmap(String url)

	// a bug in the previous versions of BitmapFactory.decodeStream may prevent
	// this code from working over a slow connection.
	// FlushedInputStream(inputStream) instead to fix the problem
	private class FlushedInputStream extends FilterInputStream {
		public FlushedInputStream(InputStream inputStream) {
			super(inputStream);
		}

		@Override
		public long skip(long n) throws IOException {
			long totalBytesSkipped = 0L;
			while (totalBytesSkipped < n) {
				long bytesSkipped = in.skip(n - totalBytesSkipped);
				if (bytesSkipped == 0L) {
					int b = read();
					if (b < 0) {
						break; // we reached EOF
					} else {
						bytesSkipped = 1; // we read one byte
					}
				}
				totalBytesSkipped += bytesSkipped;
			}
			return totalBytesSkipped;
		}
	}
}
